@@ -1,25 +1,28 @@
 # __init__.py -- Fast export/import functionality
 # Copyright (C) 2010-2013 Jelmer Vernooij <jelmer@samba.org>
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; version 2
-# of the License or (at your option) any later version of
-# the License.
+# Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
+# General Public License as public by the Free Software Foundation; version 2.0
+# or (at your option) any later version. You can redistribute it and/or
+# modify it under the terms of either of these two licenses.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# You should have received a copy of the licenses; if not, see
+# <http://www.gnu.org/licenses/> for a copy of the GNU General Public License
+# and <http://www.apache.org/licenses/LICENSE-2.0> for a copy of the Apache
+# License, Version 2.0.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA  02110-1301, USA.
 
 
 """Fast export/import functionality."""
 
+import sys
+
 from dulwich.index import (
     commit_tree,
     )
@@ -28,6 +31,9 @@ from dulwich.objects import (
     Commit,
     Tag,
     )
+from fastimport import __version__ as fastimport_version
+if fastimport_version <= (0, 9, 5) and sys.version_info[0] == 3 and sys.version_info[1] < 5:
+    raise ImportError("Older versions of fastimport don't support python3<3.5")
 from fastimport import (
     commands,
     errors as fastimport_errors,
@@ -39,8 +45,8 @@ import stat
 
 
 def split_email(text):
-    (name, email) = text.rsplit(" <", 1)
-    return (name, email.rstrip(">"))
+    (name, email) = text.rsplit(b" <", 1)
+    return (name, email.rstrip(b">"))
 
 
 class GitFastExporter(object):
@@ -53,11 +59,11 @@ class GitFastExporter(object):
         self._marker_idx = 0
 
     def print_cmd(self, cmd):
-        self.outf.write("%r\n" % cmd)
+        self.outf.write(getattr(cmd, "__bytes__", cmd.__repr__)() + b"\n")
 
     def _allocate_marker(self):
         self._marker_idx+=1
-        return str(self._marker_idx)
+        return ("%d" % (self._marker_idx,)).encode('ascii')
 
     def _export_blob(self, blob):
         marker = self._allocate_marker()
@@ -82,8 +88,10 @@ class GitFastExporter(object):
             if old_path != new_path and old_path is not None:
                 yield commands.FileRenameCommand(old_path, new_path)
             if old_mode != new_mode or old_hexsha != new_hexsha:
-                yield commands.FileModifyCommand(new_path, new_mode, marker,
-                    None)
+                prefixed_marker = b':' + marker
+                yield commands.FileModifyCommand(
+                    new_path, new_mode, prefixed_marker, None
+                )
 
     def _export_commit(self, commit, ref, base_tree=None):
         file_cmds = list(self._iter_files(base_tree, commit.tree))
@@ -148,10 +156,10 @@ class GitImportProcessor(processor.ImportProcessor):
         (author_name, author_email, author_timestamp, author_timezone) = author
         (committer_name, committer_email, commit_timestamp,
             commit_timezone) = cmd.committer
-        commit.author = "%s <%s>" % (author_name, author_email)
+        commit.author = author_name + b" <" + author_email + b">"
         commit.author_timezone = author_timezone
         commit.author_time = int(author_timestamp)
-        commit.committer = "%s <%s>" % (committer_name, committer_email)
+        commit.committer = committer_name + b" <" + committer_email + b">"
         commit.commit_timezone = commit_timezone
         commit.commit_time = int(commit_timestamp)
         commit.message = cmd.message
@@ -159,32 +167,32 @@ class GitImportProcessor(processor.ImportProcessor):
         if cmd.from_:
             self._reset_base(cmd.from_)
         for filecmd in cmd.iter_files():
-            if filecmd.name == "filemodify":
+            if filecmd.name == b"filemodify":
                 if filecmd.data is not None:
                     blob = Blob.from_string(filecmd.data)
                     self.repo.object_store.add(blob)
                     blob_id = blob.id
                 else:
-                    assert filecmd.dataref[0] == ":", \
-                        "non-marker refs not supported yet"
+                    assert filecmd.dataref.startswith(b":"), \
+                        "non-marker refs not supported yet (%r)" % filecmd.dataref
                     blob_id = self.markers[filecmd.dataref[1:]]
                 self._contents[filecmd.path] = (filecmd.mode, blob_id)
-            elif filecmd.name == "filedelete":
+            elif filecmd.name == b"filedelete":
                 del self._contents[filecmd.path]
-            elif filecmd.name == "filecopy":
+            elif filecmd.name == b"filecopy":
                 self._contents[filecmd.dest_path] = self._contents[
                     filecmd.src_path]
-            elif filecmd.name == "filerename":
+            elif filecmd.name == b"filerename":
                 self._contents[filecmd.new_path] = self._contents[
                     filecmd.old_path]
                 del self._contents[filecmd.old_path]
-            elif filecmd.name == "filedeleteall":
+            elif filecmd.name == b"filedeleteall":
                 self._contents = {}
             else:
                 raise Exception("Command %s not supported" % filecmd.name)
         commit.tree = commit_tree(self.repo.object_store,
             ((path, hexsha, mode) for (path, (mode, hexsha)) in
-                self._contents.iteritems()))
+                self._contents.items()))
         if self.last_commit is not None:
             commit.parents.append(self.last_commit)
         commit.parents += cmd.merges
