@@ -1,21 +1,22 @@
 # test_web.py -- Compatibility tests for the git web server.
 # Copyright (C) 2010 Google, Inc.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; version 2
-# of the License or (at your option) any later version of
-# the License.
+# Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
+# General Public License as public by the Free Software Foundation; version 2.0
+# or (at your option) any later version. You can redistribute it and/or
+# modify it under the terms of either of these two licenses.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# You should have received a copy of the licenses; if not, see
+# <http://www.gnu.org/licenses/> for a copy of the GNU General Public License
+# and <http://www.apache.org/licenses/LICENSE-2.0> for a copy of the Apache
+# License, Version 2.0.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA  02110-1301, USA.
 
 """Compatibility tests between Dulwich and the cgit HTTP server.
 
@@ -37,9 +38,6 @@ from dulwich.tests import (
     SkipTest,
     skipIf,
     )
-from dulwich.tests.utils import (
-    skipIfPY3,
-    )
 from dulwich.web import (
     make_wsgi_chain,
     HTTPGitApplication,
@@ -81,7 +79,6 @@ class WebTests(ServerTests):
 
 
 @skipIf(sys.platform == 'win32', 'Broken on windows, with very long fail time.')
-@skipIfPY3
 class SmartWebTestCase(WebTests, CompatTestCase):
     """Test cases for smart HTTP server.
 
@@ -91,12 +88,12 @@ class SmartWebTestCase(WebTests, CompatTestCase):
     min_git_version = (1, 6, 6)
 
     def _handlers(self):
-        return {'git-receive-pack': NoSideBand64kReceivePackHandler}
+        return {b'git-receive-pack': NoSideBand64kReceivePackHandler}
 
     def _check_app(self, app):
         receive_pack_handler_cls = app.handlers[b'git-receive-pack']
         caps = receive_pack_handler_cls.capabilities()
-        self.assertFalse(b'side-band-64k' in caps)
+        self.assertNotIn(b'side-band-64k', caps)
 
     def _make_app(self, backend):
         app = make_wsgi_chain(backend, handlers=self._handlers())
@@ -121,7 +118,6 @@ def patch_capabilities(handler, caps_removed):
 
 
 @skipIf(sys.platform == 'win32', 'Broken on windows, with very long fail time.')
-@skipIfPY3
 class SmartWebSideBand64kTestCase(SmartWebTestCase):
     """Test cases for smart HTTP server with side-band-64k support."""
 
@@ -129,8 +125,8 @@ class SmartWebSideBand64kTestCase(SmartWebTestCase):
     min_git_version = (1, 7, 0, 2)
 
     def setUp(self):
-        self.o_uph_cap = patch_capabilities(UploadPackHandler, ("no-done",))
-        self.o_rph_cap = patch_capabilities(ReceivePackHandler, ("no-done",))
+        self.o_uph_cap = patch_capabilities(UploadPackHandler, (b"no-done",))
+        self.o_rph_cap = patch_capabilities(ReceivePackHandler, (b"no-done",))
         super(SmartWebSideBand64kTestCase, self).setUp()
 
     def tearDown(self):
@@ -144,8 +140,8 @@ class SmartWebSideBand64kTestCase(SmartWebTestCase):
     def _check_app(self, app):
         receive_pack_handler_cls = app.handlers[b'git-receive-pack']
         caps = receive_pack_handler_cls.capabilities()
-        self.assertTrue(b'side-band-64k' in caps)
-        self.assertFalse(b'no-done' in caps)
+        self.assertIn(b'side-band-64k', caps)
+        self.assertNotIn(b'no-done', caps)
 
 
 class SmartWebSideBand64kNoDoneTestCase(SmartWebTestCase):
@@ -162,12 +158,11 @@ class SmartWebSideBand64kNoDoneTestCase(SmartWebTestCase):
     def _check_app(self, app):
         receive_pack_handler_cls = app.handlers[b'git-receive-pack']
         caps = receive_pack_handler_cls.capabilities()
-        self.assertTrue(b'side-band-64k' in caps)
-        self.assertTrue(b'no-done' in caps)
+        self.assertIn(b'side-band-64k', caps)
+        self.assertIn(b'no-done', caps)
 
 
 @skipIf(sys.platform == 'win32', 'Broken on windows, with very long fail time.')
-@skipIfPY3
 class DumbWebTestCase(WebTests, CompatTestCase):
     """Test cases for dumb HTTP server."""
 
